"""
This module contains functions for works video hosting


UPD: DEPRECATED
"""


from base64 import b64decode
import re
from urllib.parse import urlparse
try:
    from html.parser import unescape
except ImportError:
    from html import unescape

import warnings

# aniboom regular expressions (works after unescape method response html page)
RE_ANIBOOM = re.compile(r'"hls":"{\\"src\\":\\"(.*\.m3u8)\\"')
RE_ANIBOOM_MPD = re.compile(r'"{\\"src\\":\\"(.*\.mpd)\\"')

# kodik/anivod regular expressions
RE_KODIK_URL = re.compile(r"https://\w+\.\w{2,6}/seria/\d+/\w+/\d{3,4}p")
RE_KODIK_URL_DATA = re.compile(r'iframe.src = "//(.*?)"')
RE_KODIK_VIDEO_TYPE = re.compile(r"go/(\w+)/\d+")
RE_KODIK_VIDEO_ID = re.compile(r"go/\w+/(\d+)")
RE_KODIK_VIDEO_HASH = re.compile(r"go/\w+/\d+/(.*?)/\d+p\?")


def kodik_decoder(url_encoded: str) -> str:
    """kodik player video url decoder

    :param str url_encoded: encoded url
    :return: decoded video url"""
    warnings.warn("Function kodik_decoder is deprecated; The function "
                  "kodik_decoder is deprecated and will be removed later."
                  "use utils.Kodik.decode method",
                  category=DeprecationWarning)
    url_encoded = url_encoded[::-1]
    if not url_encoded.endswith("=="):
        url_encoded += "=="
    link: str = b64decode(url_encoded).decode()
    if not link.startswith("https"):
        link = f"https:{link}"
    return link


def kodik_parse_payload(resp: str, referer: str) -> tuple[dict, str]:
    """Parser from kodik balanser

    :param str resp: - text response html page
    :param str referer: - referer, where give this url

    :return: - tuple with data and url
    :rtype tuple:
    """
    # prepare values for next POST request
    warnings.warn("Function kodik_parse_payload is deprecated; The function "
                  "kodik_parse_payload is deprecated and will be removed later."
                  "use utils.Kodik.parse_payload method",
                  category=DeprecationWarning)
    url_data, = re.findall(RE_KODIK_URL_DATA, resp)
    type_, = re.findall(RE_KODIK_VIDEO_TYPE, url_data)
    id_, = re.findall(RE_KODIK_VIDEO_ID, url_data)
    hash_, = re.findall(RE_KODIK_VIDEO_HASH, url_data)
    data = {value.split("=")[0]: value.split("=")[1] for value in url_data.split("?", 1)[1].split("&")}
    data.update({"type": type_, "hash": hash_, "id": id_, "info": {}, "bad_user": True,
                 "ref": referer.rstrip("/")})
    return data, url_data


def get_kodik_url(raw_player_url: str):
    warnings.warn("Function get_kodik_url is deprecated; The function "
                  "get_kodik_url is deprecated and will be removed later."
                  "use utils.Kodik.parse_payload method",
                  category=DeprecationWarning)
    url_, = RE_KODIK_URL.findall(raw_player_url)
    return f"https://{urlparse(url_).netloc}/gvi"


def get_aniboom_url(raw_aniboom_response: str, *, mpd=True):
    warnings.warn("Function get_aniboom_url is deprecated; The function "
                  "get_aniboom_url is deprecated and will be removed later."
                  "use utils.Aniboom.get_url method",
                  category=DeprecationWarning)
    r = unescape(raw_aniboom_response)
    if mpd:
        return RE_ANIBOOM_MPD.findall(r)[0].replace("\\", "")
    return RE_ANIBOOM.findall(r)[0].replace("\\", "")


def is_kodik(url: str):
    """return True if player url is kodik"""
    warnings.warn("Function is_kodik is deprecated; The function "
                  "is_kodik is deprecated and will be removed later."
                  "use utils.Kodik.is_kodik method",
                  category=DeprecationWarning)
    return bool(RE_KODIK_URL.match(url))


def is_aniboom(url: str) -> bool:
    """return True if player url is aniboom"""
    warnings.warn("Function is_aniboom is deprecated; The function "
                  "is_aniboom is deprecated and will be removed later."
                  "use utils.Aniboom.is_aniboom method",
                  category=DeprecationWarning)
    return "aniboom" in url



from base64 import b64decode
import re
from urllib.parse import urlparse
from typing import NamedTuple

try:
    from html.parser import unescape
except ImportError:
    from html import unescape

from requests import Session


class AniboomM3U8Data(NamedTuple):
    quality: str
    url_suffix: str


class Aniboom:
    # aniboom regular expressions (works after unescape method response html page)
    RE_M3U8 = re.compile(r'"hls":"{\\"src\\":\\"(.*\.m3u8)\\"')
    RE_MPD = re.compile(r'"{\\"src\\":\\"(.*\.mpd)\\"')
    QUALITY = (1080, 720, 480, 360)  # works only m3u8 format
    RE_M3U8_DATA = re.compile(r'''#EXT\-X\-STREAM\-INF\:BANDWIDTH=\d+,RESOLUTION=(\d+x\d+),CODECS=".*?",AUDIO=".*?"
(.*?\.m3u8)''')  # parse master.m3u8, return [(quality, url), ...] results

    def __init__(self, session: Session):
        self.session = session
        self.headers = self.session.headers.get("user-agent")

    def get_video_url(self, player_url: str, *, quality: int = 1080, mpd: bool = False, referer: str) -> str:
        """

        :param player_url:
        :param quality:
        :param mpd:
        :param referer:
        :return:
        """
        r = self.session.get(player_url, headers={"referer": referer,
                                                  "user-agent": self.session.headers["user-agent"]})

        return self.get_aniboom_url(r.text, quality=quality, mpd=mpd)

    @staticmethod
    def get_aniboom_url(raw_aniboom_response: str, *, quality: int = 1080, mpd: bool = False) -> str:
        """

        :param quality: video quality. Available values: 480, 720, 1080
        :param raw_aniboom_response:
        :param mpd: return mpd url extension. Default False
        :return: video url
        """
        r = unescape(raw_aniboom_response)
        try:
            if mpd:
                return Aniboom.RE_MPD.findall(r)[0].replace("\\", "")
        except IndexError:
            if quality not in Aniboom.QUALITY or quality == 1080:
                return Aniboom.RE_M3U8.findall(r)[0].replace("\\", "")
            else:
                return Aniboom._set_quality(Aniboom.RE_M3U8.findall(r)[0].replace("\\", ""), quality)

    @staticmethod
    def _set_quality(m3u8_url: str, quality: int = 1080) -> str:
        """set video quality. Works only with m3u8 format

        :param m3u8_url: m3u8 url format
        :param quality: video quality. Default 1080
        :return: video url with set quality
        """
        # parse master.m3u8 response
        r = Session().get(m3u8_url, headers={"Referer": "https://aniboom.one", "Accept-Language": "ru-RU",
                                             "User-Agent":
                                                 "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) "
                                                 "AppleWebKit/537.36 (KHTML, like Gecko) "
                                                 "Chrome/94.0.4606.114 Mobile Safari/537.36"}).text
        # '640x360' - 360 '854x480' 480 '1280x720' - 720 '1920x1080' - 1080
        results = [AniboomM3U8Data(qual, url) for qual, url in Aniboom.RE_M3U8_DATA.findall(r)]
        return next((
            m3u8_url.replace("master.m3u8", data.url_suffix) for data in results if data.quality.endswith(str(quality))
        ), m3u8_url)

    @staticmethod
    def is_aniboom(url: str) -> bool:
        """return True if player url is aniboom"""
        return "aniboom" in url



"""Шаблон для добавления своих парсеров под сторонние сайты

"""
from anicli_ru.base import *
import re


class Anime(BaseAnimeHTTP):
    BASE_URL = "https://example.com"

    def search(self, q: str) -> ResultList[BaseAnimeResult]:
        """Запрос на поиск тайтла по названию.
        Здесь идёт запрос, который выведет результат поиска.

        :param str q: - строка поиска

        :return ResultList[AnimeResult]: - возвращает список объектов AnimeResult"""
        r = self.request_get().text
        return AnimeResult.parse(r)

    def ongoing(self, *args, **kwargs) -> ResultList[BaseOngoing]:
        """Вывод онгоингов
        Здесь идёт запрос, который выведет онгоинги"""
        r = self.request_get().text
        return Ongoing.parse(r)

    def episodes(self, *args, **kwargs) -> ResultList[BaseEpisode]:
        """Вывод доступных эпизодов
        Здесь идёт запрос, который выведет эпизоды с найденного Тайтла/Онгоинга

        :param Union[AnimeResult, Ongoing]: - Объект AnimeResult или Ongoing
        """
        return Episode.parse()

    def players(self, *args, **kwargs) -> ResultList[BasePlayer]:
        """Вывод доступных ссылок на видео.
        Здесь необходимо отправить запрос, на котором присутствуют ссылки на видео"""
        return Player.parse()


class AnimeResult(BaseAnimeResult):
    ANIME_HTTP = Anime()
    REGEX = {"url": re.compile("foo (.*?)"),
             "title": re.compile("bar (.*?)")}

    url: str
    title: str

    def __str__(self):
        return f"{self.title}"


class Ongoing(BaseOngoing):
    ANIME_HTTP = Anime()
    REGEX = {}

    def __str__(self):
        return


class Player(BasePlayer):
    ANIME_HTTP = Anime()
    REGEX = {"url": re.compile("url (.*?)")}
    url: str

    def __str__(self):
        return


class Episode(BaseEpisode):
    ANIME_HTTP = Anime()
    REGEX = {}

    def __str__(self):
        return ""



"""Dynamic import extractor form **extractors** directory"""
from typing import cast, Protocol, Type, Tuple, Union
import importlib
from os import PathLike, listdir

from anicli_ru.base import *


class Extractor(Protocol):
    """Typehints dyn imported extractor"""
    Anime: Type[BaseAnimeHTTP]
    AnimeResult: Type[BaseAnimeResult]
    Episode: Type[BaseEpisode]
    Ongoing: Type[BaseOngoing]
    Player: Type[BasePlayer]
    ResultList: Type[ResultList]


def all_extractors(*, absolute_directory: bool = False) -> Tuple[str, ...]:
    if __name__ != "__main__":
        dir_path = __file__.replace(__name__.split(".")[-1] + ".py", "") + "extractors"
    else:
        dir_path = "../../extractors"
    if absolute_directory:
        return tuple("anicli_ru.extractors." + _.replace(".py", "") for _ in listdir(dir_path) if
                     not _.startswith("__") and _.endswith(".py"))

    return tuple(_.replace(".py", "") for _ in listdir(dir_path) if not _.startswith("__") and _.endswith(".py"))


def import_extractor(module_name: Union[PathLike, str]) -> Extractor:
    """
    :param module_name: extractor name
    :return: Imported extractor module
    :raise ImportError:
    """
    try:
        # typehint dynamically import API extractor
        extractor = cast(Extractor, importlib.import_module(module_name, package=None))  # type: ignore
    except ModuleNotFoundError as e:
        raise ModuleNotFoundError(f"Module {module_name} has not founded") from e
    # check extractor scheme
    for class_ in ("Anime", "AnimeResult", "Episode", "Ongoing", "Player", "ResultList"):
        try:
            getattr(extractor, class_)
        except AttributeError as exc:
            raise AttributeError(f"Module {module_name} has no class {class_}. Did you import extractor?") from exc

    return extractor



from __future__ import annotations

import warnings
from collections import UserList
import re
from html import unescape
from typing import Optional, Dict, Pattern, Sequence, Union, List, TypedDict, Tuple

from requests import Session, Response

from .utils import Aniboom, Kodik

__all__ = ("BaseAnimeHTTP",
           "BaseParser",
           "BaseJsonParser",
           "BaseAnimeResult",
           "BasePlayer",
           "BaseEpisode",
           "BaseOngoing",
           "ResultList")


class TESTS(TypedDict):
    """ Dict for testing extractors

    search: title_name: str, episodes: int - check search method and get episodes

    ongoing: bool - check ongoings search

    video: bool - test get video url

    search_blocked: bool - ignore failed get episode and retry get episodes for non blocked title

    search_not_found: str title - this title has not exist

    instant: str - title, where need get all video urls

    """
    search: Tuple[str, int]
    ongoing: bool
    video: bool
    search_blocked: bool
    search_not_found: str
    instant: str


class BaseAnimeHTTP:
    """Base singleton class for send request to url, where need get html/json documents.

    the following variables and methods should be overridden in this class:

    BASE_URL: - main url site

    def search(self, q: str): - search by string

    def ongoing(self, *args, **kwargs): - search ongoings

    def episodes(self, *args, **kwargs): - search episodes

    def players(self, *args, **kwargs): - search direct video url

    _TESTS: - config dict for testing extractor

    Optional:

        USER_AGENT: - Useragent. In CLI, default set random by utils.random_agent module


    """
    BASE_URL = "https://example.com/"
    # XMLHttpRequest value required!
    USER_AGENT = {
        "user-agent":
            "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.114 Mobile Safari/537.36",
            "x-requested-with": "XMLHttpRequest"}
    _instance = None
    TIMEOUT: float = 30
    # dict for get parser config tests
    _TESTS: TESTS = {
        "search": ("experiments lain", 13),  # standard search test
        "ongoing": True,  # test search ongoings, True - yes, False - no
        "video": True,  # test get raw video, True - yes, False - no
        "search_blocked": False,  # ignore failed get episode and retry get episodes for non blocked title
        "search_not_found": "_thisTitleIsNotExist123456",  # this title has not exist
        "instant": "experiments lain"  # test instant key scroll series
    }
    # костыль для настройки поведения ключа INSTANT issue #6:
    # если сначала идёт выбор озвучки, а потом плеера, выставите значение True (see extractors/animego)
    INSTANT_KEY_REPARSE = False

    def __new__(cls, *args, **kwargs):
        # create singleton for correct store session
        if not cls._instance:
            cls._instance = super(BaseAnimeHTTP, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self, session: Session = None):
        if session:
            self.session = session
            self.session.headers.update({"x-requested-with": "XMLHttpRequest"})
        else:
            self.session = Session()
            self.session.headers.update(self.USER_AGENT)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.session.close()

    def request(self, method: str, url: str, **kwargs) -> Response:
        """Session.request method

        :param str method: method type
        :param str url: url target
        :param kwargs: optional requests.Session kwargs
        :return: requests.Response object
        """
        # context manager solve ResourceWarning (trace this in tests)
        with self.session as s:
            return s.request(method, url, timeout=self.TIMEOUT, **kwargs)

    def request_get(self, url: str, **kwargs) -> Response:
        """Session.get method

        :param str url: url target
        :param kwargs: optional requests.Session kwargs
        :return: requests.Response object
        """
        return self.request("GET", url, **kwargs)

    def request_post(self, url: str, **kwargs) -> Response:
        """Session.post method

        :param url: url target
        :param kwargs: optional requests.Session kwargs
        :return: requests.Response object
        """
        return self.request("POST", url, **kwargs)

    # need manually write requests in parsers

    def search(self, q: str) -> ResultList[BaseAnimeResult]:  # type: ignore
        """Search anime title by string pattern

        :param str q: string search
        :return: list with AnimeResult objects
        """
        raise NotImplementedError

    def ongoing(self, *args, **kwargs) -> ResultList[BaseOngoing]:  # type: ignore
        """Search ongoings

        :param args:
        :param kwargs:
        :return: list with Ongoing objects
        """
        raise NotImplementedError

    def episodes(self, *args, **kwargs) -> ResultList[BaseEpisode]:  # type: ignore
        """

        :param args:
        :param kwargs:
        :return: list with Episode objects
        """
        raise NotImplementedError

    def episode_reparse(self, *args, **kwargs):
        """Need write this method if INSTANT_KEY_REPARSE == True (see, extractors/animego.py)

        :param args:
        :param kwargs:
        :return:
        """
        raise NotImplementedError

    def players(self, *args, **kwargs) -> ResultList[BasePlayer]:  # type: ignore
        """

        :param args:
        :param kwargs:
        :return: list with Player objects
        """
        raise NotImplementedError

    def get_kodik_url(self, player_url: str, quality: int = 720, *, referer: str = "") -> str:
        warnings.warn("The function get_kodik_url is deprecated and will be removed later."
                      "use Anime.get_kodik_video method",
                      category=DeprecationWarning)
        return self.get_kodik_video(player_url, quality, referer=referer)

    def get_kodik_video(self, player_url: str, quality: int = 720, *, referer: str = "") -> str:
        """Get hls url from kodik balancer

        :param str player_url: - raw url from kodik balancer
        :param int quality: - video quality. Default 720
        :param str referer: - referer, where give this url. Default get automatically
        :return: direct video url
        """
        return Kodik(self.session).get_video_url(player_url, quality, referer=referer)

    def get_aniboom_url(self, player_url: str, quality: int = 1080) -> str:
        warnings.warn("The method get_aniboom_url is deprecated and will be removed later."
                      "use Anime.get_aniboom_video method",
                      category=DeprecationWarning)
        return self.get_aniboom_video(player_url, quality)

    def get_aniboom_video(self, player_url: str, quality: int = 1080) -> str:
        """get hls url from aniboom balancer

        :param player_url: raw url from aniboom balancer
        :return: direct video url
        """
        # fix 28 11 2021 request
        referer = self.BASE_URL if self.BASE_URL.endswith("/") else f"{self.BASE_URL}/"
        return Aniboom(self.session).get_video_url(player_url, referer=referer, quality=quality)

    def get_video(self, player_url: str, quality: int = 720, *, referer: str = "") -> str:
        """Get video from balancer. Check balancer, where from url

        :param player_url: player url from any balancer
        :param quality: video quality. Default 720
        :param referer: referer string. Default set automatically
        :return:
        """
        if "sibnet" in player_url:
            return player_url
        elif Aniboom.is_aniboom(player_url):
            url = self.get_aniboom_video(player_url, quality=quality)
            return url
        elif Kodik.is_kodik(player_url):
            url = self.get_kodik_video(player_url, quality, referer=referer)
            return url
        else:
            # catch any players for add in script
            raise TypeError(f"The link {player_url} is not defined by the available balancers.")


class ResultList(UserList):
    """Modified list object. Used in CLI for one line enumerate print elements"""

    def __init__(self):
        super().__init__()
        self.data: Sequence[BaseParser, BaseJsonParser] = []

    def print_enumerate(self, *args) -> None:
        """print elements with getattr names arg. Default invoke __str__ method"""
        if len(self) > 0:
            for i, obj in enumerate(self, 1):
                if args:
                    print(f"[{i}]", *(getattr(obj, arg) for arg in args))
                else:
                    print(f"[{i}]", obj)
        else:
            print("Results not found!")


class BaseParser:
    """Base object parser from text response

    REGEX: Dict[key: str, value: Pattern] - a dictionary with keys, and
    re.compile expressions values, which the parser will get and assign to objects

    re.compile expression must extract **one group value** from the document for the default parser to work correctly.

    If this is not the case, then you must redefine the logic of the parse classmethod
    """

    REGEX: Dict[str, Pattern]  # {"attr_name": re.compile(<regular expression>)}

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            if isinstance(v, str):
                v = int(v) if v.isdigit() else unescape(str(v))
            setattr(self, k, v)

    @classmethod
    def parse(cls, html: str) -> ResultList:
        """class object factory

        :param str html: html document
        :return: ResultList with objects
        """
        l_objects = ResultList()
        # generate dict like {attr_name: list(values)}
        results = {k: re.findall(v, html) for k, v in cls.REGEX.items()}
        for values in zip(*results.values()):
            attrs = zip(results.keys(), values)
            # generate objects like {attr_name: attr_value}
            l_objects.append(cls(**dict(attrs)))
        return l_objects


class BaseJsonParser:
    """Base parser object for JSON response (see extractors/anilibria.py)

    KEYS: Sequence - collection of keys to get from json"""
    REGEX = None
    KEYS: Sequence[str]

    @classmethod
    def parse(cls, response: Union[dict, list[dict]]) -> ResultList:
        """class object factory

        :param response: json response
        :return: ResultList with objects
        """
        rez = ResultList()
        if isinstance(response, list):
            for data in response:
                c = cls()
                for k in data.keys():
                    if k in cls.KEYS:
                        setattr(c, k, data[k])
                rez.append(c)
        elif isinstance(response, dict):
            c = cls()
            for k in response.keys():
                if k in cls.KEYS:
                    setattr(c, k, response[k])
            rez.append(c)
        return rez


BaseParserObject = BaseParser  # old alias


class BasePlayer(BaseParserObject):
    """Player object class. Contains balancer url and method get video url

    """
    ANIME_HTTP: BaseAnimeHTTP
    dub_name: str
    _player: str

    @property
    def url(self) -> str:
        """get player url"""
        return self.player_prettify(self._player)

    @staticmethod
    def player_prettify(player: str):
        """Player prettify url method and convert all named and numeric character references
        (e.g. &gt;, &#62;, &x3e;) in the string to the corresponding unicode characters:


        //foobar.com/barfoo12345 -> https://foobar.com/barfoo12345


        :param str player: - raw player url
        :return:
        """
        return f"https:{unescape(player)}"

    def get_video(self, quality: int = 720, referer: Optional[str] = None) -> str:
        """Get direct video url

        :param int quality: video quality. Default 720
        :param referer: referer string. Default set auto.
        :return: video url
        """
        if not referer:
            referer = self.ANIME_HTTP.BASE_URL if self.ANIME_HTTP.BASE_URL.endswith("/") else f"{self.ANIME_HTTP.BASE_URL}/"

        with self.ANIME_HTTP as a:
            return a.get_video(player_url=self.url, quality=quality, referer=referer)


class BaseEpisode(BaseParserObject):
    ANIME_HTTP: BaseAnimeHTTP

    def player(self) -> ResultList[BasePlayer]:  # type: ignore
        """Get list with Player object

        :return: ResultList with Player objects
        """
        with self.ANIME_HTTP as a:
            return a.players(self)


class BaseOngoing(BaseParserObject):
    ANIME_HTTP: BaseAnimeHTTP
    url: str
    title: str

    def episodes(self) -> ResultList[BaseEpisode]:  # type: ignore
        """Get list with Episode objects

        :return: ResultList with Episode objects
        """
        with self.ANIME_HTTP as a:
            return a.episodes(self)


class BaseAnimeResult(BaseParserObject):
    ANIME_HTTP: BaseAnimeHTTP
    url: str
    title: str

    def episodes(self) -> ResultList[BaseEpisode]:  # type: ignore
        """Get list with Episode objects

        :return: ResultList with Episode objects
        """
        with self.ANIME_HTTP as a:
            return a.episodes(self)



"""Kodik module utils"""
from base64 import b64decode
import re
from typing import Optional, Dict
from urllib.parse import urlparse

try:
    from html.parser import unescape
except ImportError:
    from html import unescape

from requests import Session


class Kodik:
    # kodik/anivod regular expressions
    RE_URL = re.compile(r"https://\w+\.\w{2,6}/seria/\d+/\w+/\d{3,4}p")
    RE_URL_DATA = re.compile(r'iframe.src = "//(.*?)"')
    RE_VIDEO_TYPE = re.compile(r"go/(\w+)/\d+")
    RE_VIDEO_ID = re.compile(r"go/\w+/(\d+)")
    RE_VIDEO_HASH = re.compile(r"go/\w+/\d+/(.*?)/\d+p\?")
    QUALITY = (720, 480, 360)

    def __init__(self, session: Session):
        self.session = session
        self.useragent = self.session.headers.get("user-agent")

    def get_video_url(self, player_url: str, quality: int = 720, *, referer: str = "") -> str:
        # kodik server regular expr detection
        if not self.is_kodik(player_url):
            raise TypeError(
                f"Unknown player balancer. get_video_url method support kodik balancer\nvideo url: {player_url}")
        resp = self._get_raw_payload(player_url, referer)
        # parse payload and url for next request
        data, url_data = self.parse_payload(resp, referer)
        url = self.get_api_url(player_url)

        resp = self.session.post(url, data=data, headers={
            "user-agent": self.useragent,
            "referer": f"https://{url_data}",
            "origin": url.replace("/gvi", ""),
            "accept": "application/json, text/javascript, */*; q=0.01"}).json()["links"]
        # kodik balancer returns max quality 480, but it has (720, 480, 360) values
        video_url = resp["480"][0]["src"]
        return self._get_video_quality(video_url, quality)

    def _get_raw_payload(self, player_url: str, referer: str) -> str:
        return self.session.get(player_url, headers={"user-agent": self.useragent, "referer": referer}).text

    def _get_videos_json(self, url: str, data: dict, headers: dict) -> dict:
        return self.session.post(url, data=data, headers=headers).json()["links"]

    def _get_video_quality(self, video_url: str, quality: int) -> str:
        quality = 720 if quality not in self.QUALITY else quality
        video_url = self.decode(video_url).replace("480.mp4", f"{quality}.mp4")
        # issue 8, video_url maybe return 404 code
        if self.session.get(video_url).status_code != 404:
            return video_url

        choose_quality = f"{quality}.mp4"

        for q in self.QUALITY:
            video_url = video_url.replace(choose_quality, f"{q}.mp4")
            if self.session.get(video_url).status_code == 200:
                return video_url
            choose_quality = f"{q}.mp4"
        raise RuntimeError("Video not found", video_url)

    @staticmethod
    def decode(url_encoded: str) -> str:
        """kodik player video url decoder (reversed base64 string)

        :param str url_encoded: encoded url
        :return: decoded video url"""
        url_encoded = url_encoded[::-1]
        if not url_encoded.endswith("=="):
            url_encoded += "=="
        link = b64decode(url_encoded).decode()
        if not link.startswith("https"):
            link = f"https:{link}"
        return link

    @staticmethod
    def parse_payload(resp: str, referer: str) -> tuple[dict, str]:
        """Parser from kodik balanser

        :param str resp: - text response html page
        :param str referer: - referer, where give this url

        :return: - tuple with data and url
        :rtype tuple:
        """
        # prepare values for next POST request
        url_data, = Kodik.RE_URL_DATA.findall(resp)
        type_, = Kodik.RE_VIDEO_TYPE.findall(url_data)
        id_, = Kodik.RE_VIDEO_ID.findall(url_data)
        hash_, = Kodik.RE_VIDEO_HASH.findall(url_data)
        data = {value.split("=")[0]: value.split("=")[1] for value in url_data.split("?", 1)[1].split("&")}
        data.update({"type": type_, "hash": hash_, "id": id_, "info": {}, "bad_user": True,
                     "ref": referer.rstrip("/")})
        return data, url_data

    @staticmethod
    def get_api_url(player_url: str):
        if not player_url.startswith("//"):
            player_url = f"//{player_url}"
        if not player_url.startswith("https:"):
            player_url = f"https:{player_url}"

        url_, = Kodik.RE_URL.findall(player_url)
        return f"https://{urlparse(url_).netloc}/gvi"

    @staticmethod
    def is_kodik(url: str) -> bool:
        """return True if url player is kodik"""
        return bool(Kodik.RE_URL.match(url))



from setuptools import setup, find_packages
from anicli_ru.__version__ import __version__

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='anicli-ru',
    version=__version__,
    packages=find_packages(),
    url='https://github.com/vypivshiy/ani-cli-ru',
    license='GPL-3',
    author='Georgiy aka Vypivshiy',
    author_email='',
    python_requires='>=3.7',
    description='anime grabber video api and cli tool',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['requests'],
    entry_points={
        'console_scripts': ['anicli-ru = anicli_ru.anicli:main'],
    }
)



import subprocess
import warnings


def run_player(url: str, player: str = "mpv", **commands) -> None:
    """

    :param url: hls url
    :param player: local video player. Default mpv
    :param commands: send optional commands.
        key-param="value" convert to: --key-param=value or **{"my-key-param":"c"} = --my-key-param=c
    :return:
    """
    warnings.warn("This function/method will be remove later", category=DeprecationWarning)
    if commands:
        commands = " ".join((f'--{k}="{v}"' for k, v in commands.items()))
        subprocess.run([player, url, commands])
    else:
       subprocess.run([player, url])
